/* 2008 (c) Intel Corporation,  All rights reserved.
 *
 * Original author Øyvind Kolås <pippin@openedhand.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     - Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     - Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     - Neither the name of Intel Corporation nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Intel Corporation ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL Intel Corporation BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <clutter/clutter.h>
#include "moblin-desktop.h"
#include "clutter-box2d.h"

#define BORDER_WIDTH                       4

#if 0
static ClutterActor *add_dynamic_box (ClutterActor *box2d,
                                      gint          x,
                                      gint          y,
                                      gint          width,
                                      gint          height,
                                      ClutterColor *color);
#endif
static ClutterActor * add_static_box (ClutterActor *box2d,
                                      gint          x,
                                      gint          y,
                                      gint          width,
                                      gint          height,
                                      ClutterColor *color);
static void add_static_boundary      (ClutterActor *box2d,
                                      gint          x,
                                      gint          y,
                                      gint          width,
                                      gint          height,
                                      ClutterColor *color);
static void zero_gravity             (ClutterActor *box2d)
{
  ClutterVertex gravity = {0,0};
  g_object_set (box2d, "gravity", &gravity, NULL);
}



static ClutterActor * add_static_box (ClutterActor *box2d,
                                      gint          x,
                                      gint          y,
                                      gint          width,
                                      gint          height,
                                      ClutterColor *color)
{ 
   ClutterActor *border;
   border = clutter_rectangle_new ();
   clutter_actor_set_size (border, width, height);
   clutter_actor_set_position (border, x, y);
   clutter_group_add (CLUTTER_GROUP (box2d), border);

   clutter_rectangle_set_color (CLUTTER_RECTANGLE (border), color);
   clutter_container_child_set (CLUTTER_CONTAINER (box2d), border,
                                "mode", CLUTTER_BOX2D_STATIC,
                                NULL);
   clutter_actor_set_opacity (border, 255);
   return border;
}

static void add_static_boundary (ClutterActor *box2d,
                                 gint          x,
                                 gint          y,
                                 gint          width,
                                 gint          height,
                                 ClutterColor *color)
{
  ClutterActor *top,*bottom,*left,*right;
  gint pad = BORDER_WIDTH;

  top = add_static_box    (box2d, x-pad,   y-pad, width + pad*2, pad, color);
  bottom = add_static_box (box2d, x-pad,   y+height, width + pad*2, pad, color);
  left = add_static_box   (box2d, x-pad,   y,     pad, height, color);
  right = add_static_box  (box2d, x+width, y,     pad, height, color);
}

ClutterActor *fifteen_new (const gchar *path)
{
  ClutterColor  border_color = {0x00,0xff,0xff,0x22};
  ClutterColor  white = {0xff, 0xff, 0xff, 0xff};
  ClutterColor  black = {0x00, 0x00, 0x00, 0xff};
  ClutterColor  color = {0x77,0x88,0xbb,0xaa};
  ClutterActor *box2d;
  gint          i, x,y;
  gchar        *labels[] =
    {"0","13","10","9","14","6","2","5","11","8","12","1","3","4","7","15"};

  box2d = clutter_box2d_new ();
  zero_gravity (box2d);

  add_static_boundary (box2d, 40, 40, 400, 400, &border_color);
  

  for (i=0, y=0; y<4;y++)
    for (x=0; x<4;x++, i++)
      {
        if (i!=0)
          { 
            ClutterActor *group;
            ClutterActor *rectangle;
            ClutterActor *label;
            group = clutter_group_new ();

            rectangle = clutter_rectangle_new ();
            clutter_actor_set_size (rectangle, 99, 99);
            clutter_rectangle_set_color (CLUTTER_RECTANGLE (rectangle),
                                         &color);
            clutter_container_add_actor (CLUTTER_CONTAINER (group), rectangle);

            rectangle = clutter_rectangle_new ();
            clutter_actor_set_size (rectangle, 90, 90);
            clutter_actor_set_position (rectangle, 5, 5);
            clutter_rectangle_set_color (CLUTTER_RECTANGLE (rectangle),
                                         &black);
            clutter_container_add_actor (CLUTTER_CONTAINER (group), rectangle);

            label = clutter_label_new_with_text ("Sans 10", labels[i]);
            clutter_actor_set_position (label, 40, 40);
            clutter_label_set_color (CLUTTER_LABEL (label), &white);
            clutter_container_add_actor (CLUTTER_CONTAINER (group), label);

            clutter_actor_set_position (group, x*100, y*100);
            clutter_container_add_actor (CLUTTER_CONTAINER (box2d), group);

            clutter_container_child_set (CLUTTER_CONTAINER (box2d), group,
                                         "manipulatable", TRUE,  /* user can interact with item */
                                         "mode", CLUTTER_BOX2D_DYNAMIC,
                                         NULL);
          }
      }

  clutter_box2d_set_simulating (CLUTTER_BOX2D (box2d), TRUE);

  return box2d;
}
